==> T_method-0.03.tbz: T_method-0.03/source/T_method.c <==
/*********************************************************************

                            T_method.c  
         Program for the T-method of matrix comparison 
                  following Roff et al. (1999).  
           written by Liam J. Revell <ljrevell@wustl.edu>  
    native Plan 9 port by Peter A. Cejchan  <cej@gli.cas.cz>
                                                                 
**********************************************************************/
#include <u.h>
#include <libc.h>
#include <stdio.h>

#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52744
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)

// function declarations

double ran_ecu(long *seed);
double **mem_alloc(int, int);
void get_popinfo(FILE *, char *, int *);
void get_charnum(FILE *, int *);
void get_data(FILE *, int, int, int *, double **);
int *mem_1d_int(int);
void calc_cov(int, int, int, int, int *, double **, double **);
void permute(int, int, double **, long *);
void rem_means(int,int,int,int *,double **);
double *mem_1d(int);
double calc_T(int vars, double **mat1, double **mat2);
double calc_T2(int vars, double **mat1, double **mat2);
double *mem_1d(int length);

// begin functions

double calc_T(int vars, double **mat1, double **mat2){
// this function calculates the summed absolute values of the differences between two matrices
  int i,j;
  double T;  
  T=0;
  for(i=0;i<vars;i++){
  	for(j=i;j<vars;j++){
  		T+=fabs(mat1[i][j]-mat2[i][j]);
  	}
  }
  return T;
}


double calc_T2(int vars, double **mat1, double **mat2){
// this function calculates the summed square difference between two matrices
  int i,j;
  double T2;  
  T2=0;
  for(i=0;i<vars;i++){
  	for(j=i;j<vars;j++){
  		T2+=pow(mat1[i][j]-mat2[i][j],2);
  	}
  }
  return T2;
}


double *mem_1d(int length){
// this function allocates memory to a double vector
  double *result;

  result=malloc(length*sizeof(double));
  if(!result){ printf("Memory allocation failure.\n"); exits("malloc");}

  return result;
}


void rem_means(int n1, int n2, int vars, int *id, double **data){
// this function removes population means from a data array (limit, 2 populations)
  int i,j;
  double *mean1, *mean2;
  mean1=mem_1d(vars);
  mean2=mem_1d(vars);  
  for(i=0;i<vars;i++) mean1[i]=mean2[i]=0.0;
  for(i=0;i<(n1+n2);i++){
    for(j=0;j<vars;j++){
      if(id[i]==1) mean1[j]+=data[i][j]/(double)n1;
      if(id[i]==2) mean2[j]+=data[i][j]/(double)n2;
    }
  }
  for(i=0;i<(n1+n2);i++){
    for(j=0;j<vars;j++){
      if(id[i]==1) data[i][j]-=mean1[j];
      if(id[i]==2) data[i][j]-=mean2[j];
    }
  }
}


double ran_ecu(long *seed){
// this function returns a uniform random number on the interval (0,1) using sys/time.h to seed
  int j;
  long k;
  static long seed2=123456789;
  static long iy=0;
  static long iv[NTAB];
  double result;
  
  if(*seed <= 0) {
      if((*seed)==0) *seed = 1;
      seed2=(*seed);
      for(j=NTAB+7; j>=0; j--){
	k=(*seed)/IQ1;
	*seed=IA1*(*seed-k*IQ1)-k*IR1;
	if(*seed<0) *seed += IM1;
	if(j<NTAB) iv[j]=*seed;
      }
      iy=iv[0];
  }

  k=(*seed)/IQ1;
  *seed=IA1*(*seed-k*IQ1)-k*IR1;
  if(*seed<0) *seed += IM1;
  k=seed2/IQ2;
  seed2=IA2*(seed2-k*IQ2)-k*IR2;
  if(seed2<0) seed2 += IM2;
  
  j=iy/NDIV;
  iy=iv[j]-seed2;
  iv[j]=*seed;
  if (iy<1) iy += IMM1;
  
  result=AM*iy;
  
  return(result);
}


void permute(int n, int vars, double **data, long *seed){
// this function permutes the data array at random
  int i,j,k;
  double *temp_arr;

  temp_arr=mem_1d(vars);

  for(i=n-1; i>0; i--){
    j=(i+1)*ran_ecu(seed);
    for(k=0;k<vars;k++) temp_arr[k]=data[i][k];
    for(k=0;k<vars;k++) data[i][k]=data[j][k];
    for(k=0;k<vars;k++) data[j][k]=temp_arr[k];
  }
  free(temp_arr);
}


int *mem_1d_int(int length){
// this function allocates memory to an integer vector
  int *result;

  result=malloc(length*sizeof(int));
  if(!result){ printf("Memory allocation failure.\n"); exits("malloc");}

  return result;
}


void get_popinfo(FILE *fp, char *name, int *N){
// this function gets the poupulation information from the input file
  char string[20], c;
  int i=0;
  while((string[i]=fgetc(fp))!=' ')
    i++;
  i=0;
  while((name[i]=fgetc(fp))!=' ')
    i++;
  name[i]='\0';
  i=0;
  while((string[i]=fgetc(fp))!=' ')
    i++;
  fscanf(fp,"%d",N);
}


void get_charnum(FILE *fp, int *chars){
// this function returns the number of characters in a string before a space is found
  char string[20];
  int i=0;
  while((string[i]=fgetc(fp))!=' ')
fflush(stdout);
    i++;
  fscanf(fp,"%d",chars);
}


double **mem_alloc(int width, int length){
// this function allocates memory to a double matrix
  int i;
  double **result;
  
  result=malloc(width*sizeof(double));
  if(!result){ printf("Memory allocation failure.\n"); exits("malloc");}

  for(i=0;i<width;i++){ 
    result[i]=malloc(length*sizeof(double));
    if(!result[i]){ printf("Memory allocation failure.\n"); exits("malloc");}
  }
  return result;
}


void getdata(FILE *fp, int chars, int N, int *id, double **matrix){
// this function gets the data from the input file
  int i,j;
  for(i=0;i<N;i++){
    fscanf(fp,"%d",&id[i]);
    for(j=0;j<chars;j++){
      fscanf(fp,"%lf",&matrix[i][j]);
    }
  }
}


void calc_cov(int popN, int totN, int pop, int chars, int *id, double **data, double **matrix){
// this function calculates a covariance matrix
  int i,j,k;
  double MCP, M[2];
  for(i=0;i<chars;i++){
    for(j=0;j<chars;j++){
      MCP=M[0]=M[1]=0.0;
      for(k=0;k<totN;k++){
	if(id[k]==pop){
	  MCP+=data[k][i]*data[k][j]/(double)popN;
	  M[0]+=data[k][i]/(double)popN;
	  M[1]+=data[k][j]/(double)popN;
	}
      }
      matrix[i][j]=(double)popN*(MCP-M[0]*M[1])/(double)(popN-1);
    }
  }
}

void main(){
// this is the main function
  char filename[20], pop1[10], pop2[10], c;
  int i, j, yn;
  int N1, N2, numchars, numpermute;
  int *pop;
  long int seed=-time(NULL);
  double pct=0.0, count=0.0;
  double **data, **cov_mat1, **cov_mat2;
  double P_T=0.0, P_T2=0.0;
  double T_obs, T_r;
  double T2_obs;
  FILE *fp, *dist_fp;


  // name of file with data
  fp=fopen("infile","r");
  if(!fp){
    printf("Couldn't open %s\n",filename);
    exits("open");
  }
  
  //  record T from randomization test to output file?
  yn=1;

  if(yn){
        // output filename for data from randomization distribution
  	dist_fp=fopen("outfile", "w");
  	if(!dist_fp){
  		printf("Couldn't open %s\n",filename);
                fflush(stdout);
  		exits("open");
  	}
  }

  //  desired number of randomizations for hypothesis test
  numpermute=100;
  get_popinfo(fp,pop1,&N1);
  get_popinfo(fp,pop2,&N2);
  get_charnum(fp, &numchars);

  data=mem_alloc(N1+N2,numchars);
  pop=mem_1d_int(N1+N2);

  getdata(fp,numchars,N1+N2,pop,data);

  cov_mat1=mem_alloc(numchars,numchars);
  cov_mat2=mem_alloc(numchars,numchars);

  calc_cov(N1,N1+N2,1,numchars,pop,data,cov_mat1);
  calc_cov(N2,N1+N2,2,numchars,pop,data,cov_mat2);
  
  T_obs=calc_T(numchars,cov_mat1,cov_mat2);
  T2_obs=calc_T2(numchars,cov_mat1,cov_mat2);

  rem_means(N1,N2,numchars,pop,data);

	if(yn){

		fprintf(dist_fp,"Random.  \tT        \tT^2      \n");
		fprintf(dist_fp,"obs.     \t%lf\t%lf\n",T_obs,T2_obs);
	}

  for(i=2;i<=numpermute;i++){
    
      if(yn){

    	fprintf(dist_fp,"%d",i);
    	for(j=0;j<(8-(int)log10(i));j++)
    		fprintf(dist_fp," ");
    	fprintf(dist_fp,"\t");
    }
    								  								 
    permute(N1+N2,numchars,data,&seed);

    calc_cov(N1,N1+N2,1,numchars,pop,data,cov_mat1);
    calc_cov(N2,N1+N2,2,numchars,pop,data,cov_mat2);

    T_r=calc_T(numchars,cov_mat1,cov_mat2);
    if(T_obs>T_r) P_T+=1.0/(double)numpermute;
    
     if(yn)
    	fprintf(dist_fp,"%lf\t",T_r);

    T_r=calc_T2(numchars,cov_mat1,cov_mat2);
    if(T2_obs>T_r) P_T2+=1.0/(double)numpermute;
    
    if(yn)
    	fprintf(dist_fp,"%lf\n",T_r);    
  }

  printf("\nT-method of matrix comparison \nfollowing Roff et al. (1999).   : \n");
  printf("Test:    \tStatistic:\t\t\tone-tailed p:\n");
  printf("T        \tT   = %lf\t\tP = %lf\n",T_obs,P_T);
  printf("T^2      \tT^2 = %lf\t\tP = %lf\n",T2_obs,P_T2);
}

==> T_method-0.04.tbz: T_method-0.04/source/T_method.c <==
/*********************************************************************

                            T_method.c  
         Program for the T-method of matrix comparison 
                  following Roff et al. (1999).  
           written by Liam J. Revell <ljrevell@wustl.edu>  
    native Plan 9 port by Peter A. Cejchan  <cej@gli.cas.cz>
                                                                 
**********************************************************************/
#include <u.h>
#include <libc.h>
#include <stdio.h>


#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52744
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define LENGTH 255
#define LOGFILE "logfile"
#define ITER 100

int  yn=0, numpermute=ITER;
FILE *inf, *logf, *outf;


// function declarations

double ran_ecu(long *seed);
double **mem_alloc(int, int);
void get_popinfo(FILE *, char *, int *);
void get_charnum(FILE *, int *);
void get_data(FILE *, int, int, int *, double **);
int *mem_1d_int(int);
void calc_cov(int, int, int, int, int *, double **, double **);
void permute(int, int, double **, long *);
void rem_means(int,int,int,int *,double **);
double *mem_1d(int);
double calc_T(int vars, double **mat1, double **mat2);
double calc_T2(int vars, double **mat1, double **mat2);
double *mem_1d(int length);
void T_method(void);
void usage(void);

// begin functions
void
usage(void)
{
	fprint(2, "usage: %s [-r]  [infile [outfile]]\n", argv0);
	exits("usage");
}


double calc_T(int vars, double **mat1, double **mat2){
// this function calculates the summed absolute values of the differences between two matrices
  int i,j;
  double T;  
  T=0;
  for(i=0;i<vars;i++){
  	for(j=i;j<vars;j++){
  		T+=fabs(mat1[i][j]-mat2[i][j]);
  	}
  }
  return T;
}


double calc_T2(int vars, double **mat1, double **mat2){
// this function calculates the summed square difference between two matrices
  int i,j;
  double T2;  
  T2=0;
  for(i=0;i<vars;i++){
  	for(j=i;j<vars;j++){
  		T2+=pow(mat1[i][j]-mat2[i][j],2);
  	}
  }
  return T2;
}


double *mem_1d(int length){
// this function allocates memory to a double vector
  double *result;

  result=malloc(length*sizeof(double));
  if(!result){ printf("Memory allocation failure.\n"); exits("malloc");}

  return result;
}


void rem_means(int n1, int n2, int vars, int *id, double **data){
// this function removes population means from a data array (limit, 2 populations)
  int i,j;
  double *mean1, *mean2;
  mean1=mem_1d(vars);
  mean2=mem_1d(vars);  
  for(i=0;i<vars;i++) mean1[i]=mean2[i]=0.0;
  for(i=0;i<(n1+n2);i++){
    for(j=0;j<vars;j++){
      if(id[i]==1) mean1[j]+=data[i][j]/(double)n1;
      if(id[i]==2) mean2[j]+=data[i][j]/(double)n2;
    }
  }
  for(i=0;i<(n1+n2);i++){
    for(j=0;j<vars;j++){
      if(id[i]==1) data[i][j]-=mean1[j];
      if(id[i]==2) data[i][j]-=mean2[j];
    }
  }
}


double ran_ecu(long *seed){
// this function returns a uniform random number on the interval (0,1) using sys/time.h to seed
  int j;
  long k;
  static long seed2=123456789;
  static long iy=0;
  static long iv[NTAB];
  double result;
  
  if(*seed <= 0) {
      if((*seed)==0) *seed = 1;
      seed2=(*seed);
      for(j=NTAB+7; j>=0; j--){
	k=(*seed)/IQ1;
	*seed=IA1*(*seed-k*IQ1)-k*IR1;
	if(*seed<0) *seed += IM1;
	if(j<NTAB) iv[j]=*seed;
      }
      iy=iv[0];
  }

  k=(*seed)/IQ1;
  *seed=IA1*(*seed-k*IQ1)-k*IR1;
  if(*seed<0) *seed += IM1;
  k=seed2/IQ2;
  seed2=IA2*(seed2-k*IQ2)-k*IR2;
  if(seed2<0) seed2 += IM2;
  
  j=iy/NDIV;
  iy=iv[j]-seed2;
  iv[j]=*seed;
  if (iy<1) iy += IMM1;
  
  result=AM*iy;
  
  return(result);
}


void permute(int n, int vars, double **data, long *seed){
// this function permutes the data array at random
  int i,j,k;
  double *temp_arr;

  temp_arr=mem_1d(vars);

  for(i=n-1; i>0; i--){
    j=(i+1)*ran_ecu(seed);
    for(k=0;k<vars;k++) temp_arr[k]=data[i][k];
    for(k=0;k<vars;k++) data[i][k]=data[j][k];
    for(k=0;k<vars;k++) data[j][k]=temp_arr[k];
  }
  free(temp_arr);
}


int *mem_1d_int(int length){
// this function allocates memory to an integer vector
  int *result;

  result=malloc(length*sizeof(int));
  if(!result){ printf("Memory allocation failure.\n"); exits("malloc");}

  return result;
}


void get_popinfo(FILE *fp, char *name, int *N){
// this function gets the poupulation information from the input file
  char string[LENGTH];
  int i=0;
  while((string[i]=fgetc(fp))!=' ')
    i++;
  i=0;
  while((name[i]=fgetc(fp))!=' ')
    i++;
  name[i]='\0';
  i=0;
  while((string[i]=fgetc(fp))!=' ')
    i++;
  fscanf(fp,"%d",N);
}


void get_charnum(FILE *fp, int *chars){
// this function returns the number of characters in a string before a space is found
  char string[LENGTH];
  int i=0;
  while((string[i]=fgetc(fp))!=' ')
    i++;
  fscanf(fp,"%d",chars);
}


double **mem_alloc(int width, int length){
// this function allocates memory to a double matrix
  int i;
  double **result;
  
  result=malloc(width*sizeof(double));
  if(!result){ printf("Memory allocation failure.\n"); exits("malloc");}

  for(i=0;i<width;i++){ 
    result[i]=malloc(length*sizeof(double));
    if(!result[i]){ printf("Memory allocation failure.\n"); exits("malloc");}
  }
  return result;
}


void getdata(FILE *fp, int chars, int N, int *id, double **matrix){
// this function gets the data from the input file
  int i,j;
  for(i=0;i<N;i++){
    fscanf(fp,"%d",&id[i]);
    for(j=0;j<chars;j++){
      fscanf(fp,"%lf",&matrix[i][j]);
    }
  }
}


void calc_cov(int popN, int totN, int pop, int chars, int *id, double **data, double **matrix){
// this function calculates a covariance matrix
  int i,j,k;
  double MCP, M[2];
  for(i=0;i<chars;i++){
    for(j=0;j<chars;j++){
      MCP=M[0]=M[1]=0.0;
      for(k=0;k<totN;k++){
	if(id[k]==pop){
	  MCP+=data[k][i]*data[k][j]/(double)popN;
	  M[0]+=data[k][i]/(double)popN;
	  M[1]+=data[k][j]/(double)popN;
	}
      }
      matrix[i][j]=(double)popN*(MCP-M[0]*M[1])/(double)(popN-1);
    }
  }
}


void T_method(void){

  int i, j;
  char pop1[10], pop2[10];
  int N1, N2, numchars;
  int *pop;
  long int seed=0;
  double **data, **cov_mat1, **cov_mat2;
  double P_T=0.0, P_T2=0.0;
  double T_obs, T_r;
  double T2_obs;

 
  //  desired number of randomizations for hypothesis test
  get_popinfo(inf,pop1,&N1);
  get_popinfo(inf,pop2,&N2);
  get_charnum(inf, &numchars);

  data=mem_alloc(N1+N2,numchars);
  pop=mem_1d_int(N1+N2);

  getdata(inf,numchars,N1+N2,pop,data);

  cov_mat1=mem_alloc(numchars,numchars);
  cov_mat2=mem_alloc(numchars,numchars);

  calc_cov(N1,N1+N2,1,numchars,pop,data,cov_mat1);
  calc_cov(N2,N1+N2,2,numchars,pop,data,cov_mat2);
  
  T_obs=calc_T(numchars,cov_mat1,cov_mat2);
  T2_obs=calc_T2(numchars,cov_mat1,cov_mat2);

  rem_means(N1,N2,numchars,pop,data);

	if(yn){

		fprintf(logf,"Random.  \tT        \tT^2      \n");
		fprintf(logf,"obs.     \t%lf\t%lf\n",T_obs,T2_obs);
	}

  for(i=2;i<=numpermute;i++){
    
      if(yn){

    	fprintf(logf,"%d",i);
    	for(j=0;j<(8-(int)log10(i));j++)
    		fprintf(logf," ");
    	fprintf(logf,"\t");
    }
    								  								 
    permute(N1+N2,numchars,data,&seed);

    calc_cov(N1,N1+N2,1,numchars,pop,data,cov_mat1);
    calc_cov(N2,N1+N2,2,numchars,pop,data,cov_mat2);

    T_r=calc_T(numchars,cov_mat1,cov_mat2);
    if(T_obs>T_r) P_T+=1.0/(double)numpermute;
    
     if(yn)
    	fprintf(logf,"%lf\t",T_r);

    T_r=calc_T2(numchars,cov_mat1,cov_mat2);
    if(T2_obs>T_r) P_T2+=1.0/(double)numpermute;
    
    if(yn)
    	fprintf(logf,"%lf\n",T_r);    
  }

  fprintf(outf,"\nT-method of matrix comparison \nfollowing Roff et al. (1999).   : \n");
  fprintf(outf,"Test:    \tStatistic:\t\t\tone-tailed p:\n");
  fprintf(outf,"T        \tT   = %lf\t\tP = %lf\n",T_obs,P_T);
  fprintf(outf,"T^2      \tT^2 = %lf\t\tP = %lf\n",T2_obs,P_T2);
}
void
main(int argc, char **argv)
{
	char *err;


	ARGBEGIN{
	case 'r':
                //  record T from randomization test to output file
		yn=1;
		break;
	case '?':
	default:
		usage();
		break;
	}ARGEND

	inf=stdin;
	outf=stdout;

	err = nil;
	if(yn){
        // output filename for data from randomization distribution


			logf = fopen(LOGFILE, "w");


			if(!logf){
				fprint(2, "%s: can't open %s: %r\n", argv0, LOGFILE);
				err = "open";
			}
	}

	if(argc == 1){

		inf = fopen(argv[0], "r");
			if(!inf){
				fprint(2, "%s: can't open %s: %r\n", argv0, argv[0]);
				err = "open";
			}
	}
	else if (argc == 2){

			inf = fopen(argv[0], "r");
			if(!inf){
				fprint(2, "%s: can't open %s: %r\n", argv0, argv[0]);
				err = "open";
			}
			outf = fopen(argv[1], "w");
			if(!outf){
				fprint(2, "%s: can't open %s: %r\n", argv0, argv[1]);
				err = "open";
			}
	}
	else if (argc > 2) usage();
	
	if(!err) T_method();
	exits(err);

}
